package com.cicdi.notify;

import com.cicdi.notify.template.Template;

import java.util.Map;

/**
 * 通知器
 *
 * @param <T> 通知模板
 * @author xueye
 */
public interface Notifier<T extends Template> {

    /**
     * @return 通知器唯一标识
     */
    String getNotifierId();

    /**
     * @return 通知器类型
     * @see NotifyType
     */
    NotifyType getType();

    /**
     * @return 通知提供商
     */
    Provider getProvider();

    /**
     * 指定模板ID进行发送
     *
     * @param templateId 模板唯一标识
     * @param context    上下文
     */
    void send(String templateId, Map<String, Object> context);

    /**
     * 指定模板进行发送
     *
     * @param template 通知模板
     * @param context  上下文
     */
    void send(T template, Map<String, Object> context);

    /**
     * 关闭通知器
     */
    void close();

    default <R extends Notifier<T>> R unwrap(Class<R> type) {
        return type.cast(this);
    }
}
