package com.cicdi.notify.wechat;


import com.alibaba.fastjson.JSONObject;
import com.cicdi.notify.template.Template;
import org.springframework.util.StringUtils;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.util.UriComponentsBuilder;

import java.util.Collections;
import java.util.Map;

public class WechatMessageTemplate implements Template {

    /**
     * 应用ID
     */
    private String agentId;

    private String toUser;

    private String toParty;

    private String toTag;

    private String message;

    public String getAgentId() {
        return agentId;
    }

    public void setAgentId(String agentId) {
        this.agentId = agentId;
    }

    public String getToUser() {
        return toUser;
    }

    public void setToUser(String toUser) {
        this.toUser = toUser;
    }

    public String getToParty() {
        return toParty;
    }

    public void setToParty(String toParty) {
        this.toParty = toParty;
    }

    public String getToTag() {
        return toTag;
    }

    public void setToTag(String toTag) {
        this.toTag = toTag;
    }

    public String getMessage() {
        return message;
    }

    public void setMessage(String message) {
        this.message = message;
    }

    public BodyInserters.FormInserter<String> createFormInserter(BodyInserters.FormInserter<String> inserter, Map<String, Object> context) {
        inserter.with("agentid", this.getAgentId())
                .with("msgtype","text")
                .with("text",this.createMessage(context));
        if (StringUtils.hasText(toUser)) {
            inserter.with("touser", this.createUserIdList(context));
        }
        if (StringUtils.hasText(toParty)) {
            inserter.with("toparty", this.createDepartmentIdList(context));
        }
        return inserter;

    }

    public String createJsonRequest(Map<String, Object> context){
        JSONObject json=new JSONObject();
        json.put("agentid",getAgentId());
        json.put("msgtype","text");
        // TODO
//        json.put("text",Collections.singletonMap("content",ExpressionUtils.analytical(message, context.getAllValues(), "spel")));
        json.put("text",Collections.singletonMap("content", message));

        if (StringUtils.hasText(toUser)) {
            json.put("touser", this.createUserIdList(context));
        }
        if (StringUtils.hasText(toParty)) {
            json.put("toparty", this.createDepartmentIdList(context));
        }

        return json.toJSONString();
    }


    public UriComponentsBuilder createUriParameter(UriComponentsBuilder builder, Map<String, Object> context){
        builder.queryParam("agentid", this.getAgentId())
                .queryParam("msgtype","text")
                .queryParam("text",this.createMessage(context));
        if (StringUtils.hasText(toUser)) {
            builder.queryParam("touser", this.createUserIdList(context));
        }
        if (StringUtils.hasText(toParty)) {
            builder.queryParam("toparty", this.createDepartmentIdList(context));
        }
        return builder;
    }

    public String createUserIdList(Map<String, Object> context) {
        if (StringUtils.isEmpty(toUser)) {
            return toUser;
        }
        return toUser;
//        return ExpressionUtils.analytical(toUser, context.getAllValues(), "spel");
    }

    public String createDepartmentIdList(Map<String, Object> context) {
        if (StringUtils.isEmpty(toParty)) {
            return toParty;
        }
        return toParty;
//        return ExpressionUtils.analytical(toParty, context.getAllValues(), "spel");
    }

    public String createMessage(Map<String, Object> context) {
        JSONObject json = new JSONObject();
        json.put("content", message);
//        json.put("content", ExpressionUtils.analytical(message, context.getAllValues(), "spel"));
        return json.toJSONString();
    }

}
